{ config, pkgs, lib, ... }:

{
  imports =
    [
      ./hardware-configuration.nix
      ../../configuration/common
      ../../configuration/proxmox-vm
      ../../configuration/router
    ];

  boot.loader.grub.enable = true;
  boot.loader.grub.version = 2;
  boot.loader.grub.device = "/dev/vda";

  networking.hostName = "nonat";

  networking.useDHCP = false;
  # Local Router IPs
  networking.interfaces.lo.ipv6.addresses = [
    { address = "fd00:152:152:103::1"; prefixLength = 64; }
    { address = "fd00:152:152::1"; prefixLength = 128; } # Anycast
  ];
  networking.interfaces.lo.ipv4.addresses = [
    { address = "10.152.103.1"; prefixLength = 24; }
    { address = "10.152.0.1"; prefixLength = 32; } # Anycast
  ];
  # Network
  networking.interfaces.ens18.ipv4.addresses = [
    { address = "141.24.46.169"; prefixLength = 24; }
  ];
  networking.interfaces.ens18.ipv6.addresses = [
    { address = "2001:638:904:ffca::6"; prefixLength = 64; }
  ];
  networking.defaultGateway = { address = "141.24.46.1"; interface = "ens18"; };
  networking.defaultGateway6 = { address = "2001:638:904:ffca::1"; interface = "ens18"; };
  networking.nameservers = [ "2001:638:904:ffcc::3" "2001:638:904:ffcc::4" "141.24.40.3" "141.24.40.4" ];

  networking.interfaces.ens19.ipv4.addresses = [
    { address = "192.168.10.1"; prefixLength = 24; }
  ];
  networking.nat = {
    enableIPv6 = true;
    enable = true;
    externalInterface = "ens18";
    externalIPv6 = (lib.head config.networking.interfaces.ens18.ipv6.addresses).address;
    externalIP = (lib.head config.networking.interfaces.ens18.ipv4.addresses).address;
    internalInterfaces = [ "ens19" ];
    internalIPv6s = [ "fd00:152:152::/48" ];
    internalIPs = [ "10.152.0.0/16" ];
  };

  networking.wireguard.enable = true;
  networking.wireguard.interfaces = {
    wg-porter = {
      ips = [ "fe80::1337:2/64" "169.254.137.2/24" ];
      peers = [ {
          allowedIPs = [ "0.0.0.0/0" "::/0" ];
          endpoint = "porter.net.clerie.de:51337";
          publicKey = "TzQV60SvPZuJ9yTzvyGwejyXw1SlKkilS4UUvrQcyGk=";
      } ];
      listenPort = 51337;
      allowedIPsAsRoutes = false;
      privateKeyFile = "/var/src/secrets/wireguard/wg-porter";
    };
  };

  networking.firewall.allowedUDPPorts = [ 51337 ];

  petabyte.policyrouting = {
    enable = true;
    rules6 = [
      { rule = "from all to fd00:152:152::/48 lookup 1000"; prio = 10000; } # OSPF
      { rule = "from fd00:152:152::/48 lookup 2000"; prio = 10000; } # BGP
      { rule = "from all to fd00:152:152::/48 unreachable"; prio = 20000; }
    ];
    rules4 = [
      { rule = "from all to 10.152.0.0/16 lookup 1000"; prio = 10000; } # OSPF
      { rule = "from 10.152.0.0/16 lookup 2000"; prio = 10000; } # BGP
      { rule = "from all to 10.152.0.0/16 unreachable"; prio = 20000; }
    ];
  };

  services.bird2.enable = true;
  services.bird2.config = ''
  router id ${ (lib.head config.networking.interfaces.lo.ipv4.addresses).address };

  ipv6 table ospf6;
  ipv4 table ospf4;

  protocol direct direct_lo {
    interface "lo";
    ipv6 {
      table ospf6;
    };
    ipv4 {
      table ospf4;
    };
  }

  protocol direct direct_ens19 {
    interface "ens19";
    ipv6 {
      table ospf6;
    };
    ipv4 {
      table ospf4;
    };
  }

  protocol kernel kernel_ospf6 {
    kernel table 1000;
    ipv6 {
      table ospf6;
      export filter {
        krt_prefsrc=${ (lib.head config.networking.interfaces.lo.ipv6.addresses).address };
        accept;
      };
      import none;
    };
  }

  protocol kernel kernel_ospf4 {
    kernel table 1000;
    ipv4 {
      table ospf4;
      export filter {
        krt_prefsrc=${ (lib.head config.networking.interfaces.lo.ipv4.addresses).address };
        accept;
      };
      import none;
    };
  }

  protocol ospf v3 ospf_6 {
    ipv6 {
      table ospf6;
      import all;
      export all;
    };
    area 0 {
      interface "wg-porter" {
        cost 80;
        type pointopoint;
      };
    };
  }

  protocol ospf v3 ospf_4 {
    ipv4 {
      table ospf4;
      import all;
      export all;
    };
    area 0 {
      interface "wg-porter" {
        cost 80;
        type pointopoint;
      };
    };
  }

  protocol device {
    scan time 10;
  }
  '';

  system.stateVersion = "21.03";
}
